#define _BUILT_IN_C

#ifndef _UNISTD_H
#include <unistd.h>
#endif

#ifndef _LIMITS_H
#include <limits.h>
#endif

#ifndef _STDLIB_H
#include <stdlib.h>
#endif

#ifndef _SIGNAL_H
#include <signal.h>
#endif

#ifndef _STRING_H
#include <string.h>
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#ifndef _STDIO_H
#include <stdio.h>
#endif

#ifndef _DIRENT_H
#include <dirent.h>
#endif

#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

#ifndef _SYS_STAT_H
#include <sys/stat.h>
#endif

#ifndef MAX_CMDLINE_LENGTH
#define MAX_CMDLINE_LENGTH 1024
#endif

int built_in(char **argv)
{
		int index;
		extern char **environ;
		extern logical EXIT_COMMAND;
		char *cwd;
		
		cwd=get_current_dir_name();		/* "get_current_dir_name" alloca automaticamente
														memoria sufficiente a contenere la stringa */
		
		if (!strcmp(argv[0], "exit")) {	/* Comando di uscita */
				EXIT_COMMAND=TRUE;
				return(1);
		}

		if (!strcmp(argv[0], "cd")) {		/* Cambia CWD */
		
				if (argv[1] != NULL) {
						chdir(argv[1]);
				}
				else{
						chdir(getenv("HOME"));
				}
				return(1);
				
		}
		
		if (!strcmp(argv[0], "env")) {	/* Stampa l'Environment */
				
				index=0;

				printf("--Environment--\n");
				while (environ[index] != NULL) {
						printf("%s\n", environ[index++]);
				}
				return(1);
				
		}

		if (!strcmp(argv[0], "cwd")) {	/* Stampa il pathname della CWD */
		
				printf("Current Work Directory: %s\n", cwd);
				return(1);
		}

		if (!strcmp(argv[0], "PS")) {		/* Lista i processi in esecuzione */
				  show_pprocs(PPROCS_STRUCT_REFERENCE, ALL);
				  return(1);
		}

		if (!strcmp(argv[0], "back")) {	/* Esegue il programma in BackGround */
				  if ( argv[1] != NULL ) {
							 argv_shift_left(argv);
							 execute_background(argv);
				  }
				  else {
							 printf("Must digit an executable!\n");
				  }
				  return(1);
		}

		if (!strcmp(argv[0], "reco")) {	/* Esegue ricorsivamente il programma */
				  if ( argv[1] != NULL ) {
							 argv_shift_left(argv);
							 if ( reco(argv,cwd) != 1)
										printf("\n---Reco ERROR---\n");
							 chdir(cwd);
				  }
				  else
							 printf("Must digit an executable!\n");
		}

		return(-1);		/* Non  stato fatto nulla... */
}
/*------------------------------------- exec_pipe */
/* In base all'array di comandi costruito da "cmd_tockenizer",
	crea le pipe, crea i processi figli, attacca i descrittori
	delle pipe agli Standard Input e Output, esegue i processi,
	attende la terminazione dell'ultimo della pipeline

INPUT:
	array dei comandi
OUTPUT:
	nulla. In caso di errore stampa un messaggio
	*/
void exec_pipe(char** parsed_processes)
{
		  /* Questa variabile locale viene impostata da
			  "cmd_tokenizer" */
		  extern int number_processes_to_execute;
		  
		  int **pipes_fd;
		  
		  pid_t pid;
		  char **argv_p;
		  int status;

		  int index1, index2; /* indici di supporto */

		  /* Alloca memoria per gli interi dei descrittori
			  e apre i descrittori delle PIPE */
		  pipes_fd=(int**)malloc(number_processes_to_execute * sizeof(int*));
		  for ( index1=0; index1 <= number_processes_to_execute-2; index1++) {
					 pipes_fd[index1]=(int*)malloc(2 * sizeof(int)); /* Allocazione Memoria */
					 if (pipe(pipes_fd[index1]) != 0) {	/* Apertura dei descrittori */
								perror("\n---Pipe ERROR---\n");
								exit(EXIT_FAILURE);
					 }
		  }
		  	
		  for (index1=0; index1 <= number_processes_to_execute-1; index1++) {
					 if ( (pid=fork()) <0 ) {
								perror("\n---Fork ERROR---\n");
								exit(EXIT_FAILURE);
					 }
					 else{
								if ( pid > 0 ) {
										  /* Se si tratta dell'ultimo processo, lo attende...*/
										  if ( index1 == (number_processes_to_execute-1) ) {
													 printf("Must digit CTRL+C to continue\n");
													 add_and_wait(parsed_processes[index1], pid);
										  }
								}
								else {
										  if ( pid == 0 ) {
													 /* Se non  il primo processo */
													 if ( index1 != 0 ) 
																dup2(pipes_fd[index1-1][0], STDIN_FILENO);
													 
													 /*Se non  l'ultimo processo */
													 if ( index1 != (number_processes_to_execute -1) )
																dup2(pipes_fd[index1][1], STDOUT_FILENO);
													 /* Chiude tutti i descrittori inutili */
													 for ( index2=0; index2 <= number_processes_to_execute-2; index2++) {
																close(pipes_fd[index2][0]);
																close(pipes_fd[index2][1]);
													 }
														
													 /*Come al solito, parsa la linea di comando e quindi esegue */
													 if ( (argv_p=cmd_line_parser(parsed_processes[index1])) == NULL) {
																perror("\n---Memory ERROR---\n");
																exit(EXIT_FAILURE);
													 }

													 execvp(argv_p[0], argv_p);
													 perror("\n---Exec ERROR---\n");
													 exit(EXIT_FAILURE);
										  }
								}
					 }
		  }
		  /* Resetta il terminale alla fine della PIPE*/
		  /* Purtroppo permangono dei problemi...*/
		  system("reset -Q");
}
/*------------------------ reco */
/* Esegue il comando "arg_v" per tutto
	il sottoalbero radicato in "path_dir".

INPUT:
"argv_p" comando da eseguire (e argomenti)
"path_dir" directory da cui partire

OUTPUT:
ritorna "1" in caso di successo
 "-1" altrimenti*/
int reco(char **argv_p, char *path_dir)
{
		  DIR* cur_directory;
		  struct dirent *cur_dentry;
		  struct stat cur_dentry_info;
		  char temp_path[MAX_CMDLINE_LENGTH];
			
		  /* Cambia directory corrente */
		  chdir(path_dir);

		  /* Esegue il comando richiesto nella CWD */
		  if ( execute_and_wait(argv_p) != 1)
					 return (-1);
		
		  cur_directory=opendir(path_dir);
		
		  /* Queste due "chiamate a vuoto" di "readdir" servono ad
			  evitare "." e "..", e quindi dei loop*/
		  cur_dentry=readdir(cur_directory);	
		  cur_dentry=readdir(cur_directory);

		  while ( (cur_dentry=readdir(cur_directory)) != NULL ) {
					 /* Compone la stringa del PATH_NAME della nuova directory */
					 strcpy(temp_path,path_dir);
					 strcat(temp_path,"/");
					 strcat(temp_path,cur_dentry->d_name);

					 /* Ricava le informazioni necessarie sulla ENTRY...*/
					 if (stat(temp_path, &cur_dentry_info) == -1)
								return (-1);

					 /* Ovviamente, se  una directory richiama RECO */
					 if (S_ISDIR(cur_dentry_info.st_mode))
								reco(argv_p, temp_path);
		  }

		  closedir(cur_directory);

		  return(1);
}
		  
