#define _CMD_LINE_PARSER_C

#ifndef _STRING_H
#include <string.h>
#endif

#ifndef _STDLIB_H
#include <stdlib.h>
#endif

/* Prototipi delle funzioni =============================*/
/* AUSILIARIE */
logical empty_string(char *string);
void array_str_free(char** str);
void argv_shift_left(char** argv);

/* PRINCIPALI */
char **cmd_line_parser(char *string_cmd);
char **cmd_tokenizer(char *string_cmd);
/*=======================================================*/

/*======================================Funzioni principali */
/*-----------------------------------cmd_line_parser */
/* Esegue un semplice "parsing" della linea di comando.
   -Prende in ingresso una stringa ed usa il carattere ' ' (spazio) come delimitatore.
   -Restituisce un Array di Puntatori a Char (un "Array di Array di caratteri") utile come parametro per EXECVE

   "*string_cmd" Riga di comando letta dalla SHELL

   In caso di errore nell'allocazione della memoria restituisce NULL

note: 
	-E' previsto che il "carattere di terminazione" della stringa sia '\0'
	-Una stringa "vuota"  costituita dal carattere '\0' e non '\n'
	-In caso di stringa "vuota" ritorna "arg_vector[1 solo elemento] con arg_vector[0] == NULL"
*/
char **cmd_line_parser(char *string_cmd)
{
   char **arg_vector=NULL; 	/* Il puntatore al vettore di ARGOMENTI. 
				   E' inizializzato a NULL perch richiesto dalla f. REALLOC */
   char *temp;			/* Puntatore di appoggio */
	
   int arg_number=0;		/* Numero di argomenti...*/	
   int word_length;		/* Lunghezza della parola "parsata" */
	
   if ( strlen(string_cmd) > 0 ){	/* Se la stringa risulta pi lunga di 0 caratteri */
			
      temp=strtok(string_cmd, " ");	/* prima scansione per la ricerca del TOKEN */
			
      while(temp != NULL){
	 		word_length=strlen(temp);	/* alloca solo la memoria necessaria */
		
	 		arg_vector=(char**)realloc(arg_vector, (++arg_number) * sizeof(char*)); /* Allocazione e controllo di errore */
	 		if(arg_vector == NULL) 
	    		return(NULL);
		
	 		arg_vector[arg_number -1]=(char*)malloc((word_length +1) * (sizeof(char)));	/* come sopra... */
	 		if(arg_vector[arg_number -1] == NULL) 
	    		return(NULL);
		
	 		strcpy(arg_vector[arg_number -1], temp);     /* Copia dei dati nell'ARRAY */
		
	 		temp=strtok(NULL, " ");		      /* Aggiornamento del puntatore TEMP */
      }
   }
	
   /* Inserimento dell'ultimo valore NULL nell'ARRAY */
   arg_vector=(char**)realloc(arg_vector, (arg_number+1) * sizeof(char*));	
   if(arg_vector == NULL) 
      return(NULL);
   arg_vector[arg_number]=NULL;

   return(arg_vector);
	
}
/*--------------------------------------------*/

/*--------------------------------------------cmd_tokenizer */
/* Suddivide una stringa in un "array di stringhe" suddividendole
   in base al carattere PIPE "|".

   In caso di "stringhe vuote" vengo AUTOMATICAMENTE omesse.
   Si intendono "vuote" quelle stringhe composte solo da caratteri " " (spazi).

   Per spiegazioni pi dettagliate rifarsi alla funzione "cmd_line_parser" che risulta
   molto simile */  
char **cmd_tokenizer(char *string_cmd)
{
   extern int number_processes_to_execute;	/* dichiarata nel MAIN */
	char **p_vector=NULL; 	
   char *temp;		
   int p_number=0;	
   int word_length;
	
   if ( strlen(string_cmd) > 0 ){
      temp=strtok(string_cmd,"|");	
      while(temp != NULL){
	 		if (empty_string(temp) == FALSE) {  /* ...se la stringa  "vuota" viene omessa */
	    		word_length=strlen(temp);
	    		if ( (p_vector=(char**)realloc(p_vector, (++p_number) * sizeof(char*))) == NULL)
					return(NULL); 
	    		
				if ( (p_vector[p_number -1]=(char*)malloc((word_length +1) * (sizeof(char)))) == NULL)
					return(NULL);	
	       
			 	strcpy(p_vector[p_number -1], temp);     
	 		}
	 		temp=strtok(NULL,"|");		    
      }
   }
   
	number_processes_to_execute=p_number;	/* Imposto la variabile globale */
	if ( (p_vector=(char**)realloc(p_vector, (p_number+1) * sizeof(char*))) == NULL)
		return(NULL);	
   p_vector[p_number]=NULL;
   return(p_vector);
}

/*=========================================Funzioni ausiliarie */
/*-----------------------------------empty_string---*/
/* Controlla se la stringa "string"  costituita solo da caratteri " "
   o meno.
   Ritorna "TRUE" in caso affermativo. "FALSE" altrimenti*/

/* nota bene:
	richiede sia gi definito il tipo "logical".
	"typedef enum{FALSE,TRUE} logical;" (guardare _PSH_MAIN) */
	
logical empty_string(char *string)
{
   int length=strlen(string);
   logical empty=TRUE;
   int index=0;

   for (index=0; index<=length-1; index++) {
      if (string[index] != 32)
	 		empty=FALSE;
   }
   return(empty);
}

/*--------------------------------------------array_str_free*/
/* Pulisce la memoria dal VETTORE di argomenti*/

/* nota bene:
	Richiede che l'array abbia come ultimo elemento una stringa=NULL*/
void array_str_free(char** argv)
{
   char *temp;
   int index=0;
		
   temp=argv[index];
   while ( temp != NULL ) {
      free(temp);
      temp=argv[++index];
   }
		
   free(argv);
}
/*--------------------------------------------*/

/*--------------------------------------------argv_shift_left */
/* Esegue uno SHIFT a sinistra degli elementi dell'array
	di stringhe in INPUT.
	Serve per "sistemare l'input" ai comandi BUILT-IN */

void argv_shift_left(char** argv)
{
		  int index=0;

		  while (argv[index] != NULL)
					 argv[index]=argv[++index];
}
