#define _PSH_MAIN
/*
	File principale della Patty Shell (PSH).
	
Studente: Ivan De Marino
Matricola: 50/529
esame: Laboratorio di Sistemi Operativi mod. B a.a. 2002/2003
*/

/*-----------------------HEADER standard*/
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#ifndef _PPROCS_STRUCT_FUNCTION_C
#include "pprocs_struct_function.c"
#endif

/*===========================Variabili GLOBALI*/
typedef enum{FALSE,TRUE} logical;

PPROCS *PPROCS_STRUCT_REFERENCE;	/* Puntatore alla struttura PPROCS utilizzata per
									   		conservare varie informazioni sui processi mandati in
									   		esecuzione da PSH */

pid_t sh_pid;							/* PID della shell */
pid_t fg_pid;							/* PID del processo in ForeGround */

logical sigtstp;						/* Serve a segnalare una situazione particolare: 
							   				processo stoppato con SIGTSTP */

logical EXIT_COMMAND;				/* Variabile logica che controlla l'eventuale terminazione di PSH */

char **processes_to_execute;		/* Array di stringhe che contiene tutti i programmi da eseguire */
int number_processes_to_execute;	/* Numero dei programmi da eseguire */
/*============================================*/

#ifndef _SIGNAL_AND_EXECS_C
#include "include/signal_and_execs.c"
#endif

#ifndef _CMD_LINE_PARSER_C 
#include "include/cmd_line_parser.c"
#endif

#ifndef _BUILT_IN_C
#include "include/built_in.c"
#endif

#define MAX_CMDLINE_LENGTH 1024

int main(int argc, char *argv[], char *envp[])
{
/*=========== Variabili utilizzate ================================================================================*/
   char cmdline[MAX_CMDLINE_LENGTH];		/* Array di CHAR per la memorizzazione della linea di comando */
	
   char** arg_vector;		/* Puntatore all'area di memoria che contiene gli argomenti "parsati" dalla linea di comando */
	
   EXIT_COMMAND=FALSE;		/* Verifica la condizione di uscita: l'inserimento del comano "exit" */	
   sigtstp=FALSE;				/* Verifica se  stato stoppato un processo con il segnale SIGTSTP */
	
/*=========== Inizializzazione ====================================================================================*/	
/* Memorizzazione dei pid fondamentali  */
   sh_pid=fg_pid=getpid();	 /* "sh_pid" e "fg_pid" coincidono inizialmente... */

/* Redirezione dei segnali */
   psh_signal_redirection();	
	
/* Inizializzazione delle strutture utilizzate dalla PSH */
   PPROCS_STRUCT_REFERENCE=create_PPROCS();	/* creazione ed inizializzazione struttura PPROCS */

/* la PSH  il primo processo di cui
   si conservano le informazioni */
   add_pprocs("PSH", getpid(), getppid(), ACTIVE, PPROCS_STRUCT_REFERENCE);

/*=========== CODE START ============*/
	
   printf("\nPatty SHELL READY");
   while (!EXIT_COMMAND) {
      printf("\nPSH:%s> ", get_current_dir_name());	/* PROMPT */
      fgets(cmdline, MAX_CMDLINE_LENGTH, stdin);	
      cmdline[strlen(cmdline)-1]='\0';						/* Questo "ritocco" alla linea di comando influir su EXEC
																			che non accetta caratteri "\n" tra gli argomenti */
		
		number_processes_to_execute=0;		/* Inizialmente non esistono processi da eseguire...
															E' una variabile globale */
		
		/* Viene "parsata" tutta la linea di comando digitata, cos da verificare quanti e quali processi eseguire
			e, in pi, evidenziare eventuali PIPE da creare tra questi processi.
			Serve anche a risolvere eventuali errori di digitazione */
		if ( (processes_to_execute=cmd_tokenizer(cmdline)) == NULL) {	
				  perror("\n---Memory ERROR---\n");
				  exit(EXIT_FAILURE);
		}

		strcpy(cmdline, "\0");		/* Svuota la Command Line */

		if (processes_to_execute[0] != NULL) {		/* C' almeno un comando da eseguire */
			if (processes_to_execute[1] == NULL) {		/* C' un solo comando da eseguire */
						
					  /* Ora invece "parsa" la stringa del singolo processo, come parte del risultato
						  del primo parsing. Questo permette un riutilizzo modulare e stratificato del tutto */
					  if ( (arg_vector=cmd_line_parser(processes_to_execute[0])) == NULL) {	
								 perror("\n---Memory ERROR---\n");
								 exit(EXIT_FAILURE);
					  }
				
					  if (arg_vector[0] != NULL) {		/* Se la stringa non  vuota */
								 if (built_in(arg_vector) == -1) {  /* Se non sono stati eseguiti Built-IN command... */
											if ( execute_and_wait(arg_vector) == -1 )		/* Parte l'esecuzione di un processo */
													  printf("\n---Normal Execution ERROR---\n");
								 }
								 array_str_free(arg_vector);		/* Questa funzione ripulisce la memoria occupata */
					  }
			}
			else {
					  /* Siccome sono stati rilevati pi di un processo da eseguire,
						  viene richiamata la funzione per l'esecuzione dei processi in PIPE */
					  exec_pipe(processes_to_execute);	
			}
			array_str_free(processes_to_execute);		/* ...come sopra... */
		}
		wait_childs();		/* Ad ogni esecuzione "recupera" tutti 
									i processi figli ASINCRONI eventualmente terminati */
	}
}
/*============== CODE END ============*/
