#define _PPROCS_STRUCT_FUNCTION_C

#ifndef _PPROCS_STRUCT_C
#include "pprocs_struct.c"
#endif

/*=======================     Strutture utili alla gestione dei processi     =============
  Con queste funzioni  possibile tenere traccia dei processi eseguiti, del loro stato
  (ovvero se sono ATTIVI o sono BLOCCATI), informazioni come PID e PPID, nome del processo (ovvero
  la riga di comando che l'ha richiamato). */

/*---PROTOTIPI delle FUNZIONI:-----*/

PPROCS *create_PPROCS(void);
void add_pprocs(char *nname, pid_t npid, pid_t nppid, int stat, PPROCS *HEAD);
int show_pprocs(PPROCS *HEAD, const int to_vis);
pproc_str* find_pprocs(pid_t pid, PPROCS *HEAD);
int remove_pprocs(pid_t rpid, PPROCS *HEAD);
/* --------------------------------------------------*/

#ifndef _STDLIB_H
#include <stdlib.h>
#endif

#ifndef _STRING_H
#include <string.h>
#endif

#ifndef _STDIO_H
#include <stdio.h>
#endif 

/* Queste costanti sono utili per la "classificazione" dei processi */
#define ALL 0		
#define ACTIVE 1
#define BLOCKED 2

/*-------------------------------------- create_pprocs */
/*Inizializza la struttura PPROCS */
PPROCS* create_PPROCS(void)
{
	PPROCS *new;
	new=(PPROCS*)malloc(sizeof(PPROCS));
	new->pprocs_num=0;
	new->first_pproc=NULL;
	new->last_pproc=NULL;
	return(new);
}

/*--------------------------------------- add_pprocs */
/* Aggiunge un nuovo elemento alla struttura PPROCS*/

/* "*nname"  il nome del processo 
   "npid" il suo PID
   "nppid" il suo PPID
   "stat" il suo "stato iniziale" (ovvero Attivo o Bloccato)
   "*HEAD" la struttura PPROCS a cui aggiungerlo. Deve essere stata inizializzata da "create_PPROCS"*/
void add_pprocs(char *nname, pid_t npid, pid_t nppid, int stat, PPROCS *HEAD)
{
	/*Allocazione memoria*/
	pproc_str *new;
	new=(pproc_str*)malloc(sizeof(pproc_str));

	/*Inserimento informazioni del nuovo processo*/
	strcpy(new->proc_name, nname);
	new->pid=npid;
	new->ppid=nppid;
	new->status=stat;
	
	/*Aggiornamento puntatori della struttura dati PPROCS
	  Il nuovo elemento viene inserito in testa alla struttura*/
	
	/*Controlla se  il primo processo ad essere creato. Se si, opera di conseguenza...*/
	if(HEAD->pprocs_num == 0)
		HEAD->last_pproc=new;		/*...il nuovo  primo,corrente ed ultimo nella lista*/
	else
		HEAD->first_pproc->prec_pproc=new;	/*...il processo che prima era in testa alla lista, 
							  deve aggiornare il suo puntatore al precedente*/

	new->next_pproc=HEAD->first_pproc;
	new->prec_pproc=NULL;

	HEAD->first_pproc=new;
	
	HEAD->pprocs_num++;
}

/*------------------------------------- show_pprocs */
/*Mostra le informazioni dei processi presenti nella struttura.
  Permette di selezionare se visualizzare tutti
  oppure solo quelli ACTIVE (attivi)
  oppure solo quelli BLOCKED (bloccati)*/
  
/* "*HEAD"  la struttura PPROCS di cui si vogliono listare gli elementi
   "to_vis" serve a specificare "cosa visualizzare":
   o ALL
   o ACTIVE
   o BLOCKED*/
/* Ritorna "1" se ci sono elementi da visualizzare. "0" altrimenti.*/
int show_pprocs(PPROCS *HEAD, const int to_vis)
{
	/* Questa variabile ci aiuta a raccogliere ulteriori informazioni: il numero di processi visualizzati */
	int showned=0;
	pproc_str *support;
	
	/* Verifica...*/
	if(HEAD->pprocs_num == 0)
		return(0);		/* Errore: Non ci sono processi */
	
	
	/*Aggiornamento puntatore al corrente...*/
	support=HEAD->first_pproc;

	/* O si visualizzano TUTTI i processi, o solo quelli con la variabile "status" impostata a "to_vis" */
	/* ...intestazioni...*/
	if(to_vis == ALL) printf("\n---   ALL process   --- status 1=active / 2=blocked\n");
	else if(to_vis == ACTIVE) printf("\n---   ACTIVE process   --- status 1=active / 2=blocked\n");
		else printf("\n---   BLOCKED process   --- status 1=active / 2=blocked\n");
	
	/* Il ciclo controlla ogni volta se esistono altri processi o se la lista  finita...*/
	do{
		if(support->status == to_vis || to_vis==ALL){  /* Controllo incrociato */
			printf("PID: %d\t PPID: %d\t -\t P_NAME: %s\t STATUS: %d\n", support->pid, support->ppid, support->proc_name, support->status);
			showned++;
		}
	}while((support=support->next_pproc) != NULL);
	
	/* Informazioni addizionali...*/
	printf("Total Process: %d\t -\tShowned Process: %d\n", HEAD->pprocs_num, showned);	

	return(1);
}

/*------------------------------- find_pprocs */
/* Cerca un processo all'interno della struttura PPROCS in base al PID
   se presente, ritorna un puntatore alla struttura "pproc" di quel processo, altrimenti "NULL"
   Il puntatore "HEAD->current_pproc" punter all'elemento, se trovato. Altrimenti a "NULL"*/

/* "pid"  il PID da cercare;
   "*HEAD"  la struttura PPROCS in cui cercare*/
pproc_str* find_pprocs(pid_t pid, PPROCS* HEAD)
{
	/* Variabile di appoggio */
	int founded=0;					/* serve come variabile booleana per verificare 
									   se l'elemento  stato trovato */
	pproc_str *current_pointed;		/* puntatore utile alla ricerca */
	
	
	current_pointed=HEAD->first_pproc;		/* ...punta al primo elemento... */
	/*Il ciclo continua finch non sono finiti gli elementi 
	  oppure  stato trovato il processo cercato*/
	do{
			if (current_pointed->pid == pid) founded=1;
			else current_pointed=current_pointed->next_pproc;
	
	}while( (current_pointed != NULL) && (!founded) );

	/*Valori ritornati*/
	if(founded) return(current_pointed);
	else return(NULL);
}

/*----------------------------------------------- remove_pprocs */
/* Cerca e, se trovato, rimuove dalla lista il processo con PID = "rpid".
   Ritorna "1" se lo trova e lo elimina. Altrimenti "-1".*/

/* "rpid"  il PID da rimuovere
   "*HEAD"  la struttura PPROCS in cui cercare*/
/* nota: Dopo l'uso di questa funzione "HEAD->current_pproc" punter a "HEAD->first_pproc"*/

int remove_pprocs(pid_t rpid, PPROCS* HEAD)
{
	int cond1, cond2;				/* servono solo a semplificare il codice: 
									   conservano le "condizioni da controllare e combinare " */
	pproc_str *current_founded;		/* Punter alla struttura "pprocs" trovata da find che si vuole eliminare */
	
	if((current_founded=find_pprocs(rpid,HEAD)) != NULL){	/* Se l'elemento  presente...*/
		
		/* "Salvo" le condizioni da controllare in due variabili intere per comodit */
		cond1=(current_founded->prec_pproc == NULL);
		cond2=(current_founded->next_pproc == NULL);

		if(!cond1 && !cond2){
			/* E' un elemento "interno" alla lista...*/
			current_founded->prec_pproc->next_pproc=current_founded->next_pproc;
			current_founded->next_pproc->prec_pproc=current_founded->prec_pproc;
		}else{ 
			if(cond1 && cond2){
				/* E' l'unico elemento presente... */
				HEAD->first_pproc=HEAD->last_pproc=NULL;
			}else
				if(cond1 && !cond2){
					/* E' il primo elemento della lista... */
					current_founded->next_pproc->prec_pproc=NULL;
					HEAD->first_pproc=current_founded->next_pproc;
				}else{
					/* E' l'ultimo elemento della lista... */
					current_founded->prec_pproc->next_pproc=NULL;
					HEAD->last_pproc=current_founded->prec_pproc;
				}
		}
		
		free(current_founded);	/* libera la memoria del "pproc" eliminato */
				
		HEAD->pprocs_num--;
		return(1);
	}else return(0);
}

/*------------------------------------------- ch_status_pprocs */
/* Modifica lo "status" di esecuzione di un processo:
	da ACTIVE a BLOCKED
	da BLOCKED ad ACTIVE

INPUT:
	pid
	puntatore alla testa della struttura PPROCS
OUTPUT:
	"1" se lo trova
	"-1" altrimenti */
int ch_status_pprocs(pid_t pid, PPROCS* HEAD)
{
		pproc_str *current_founded;

		if ( (current_founded=find_pprocs(pid, HEAD)) != NULL ){	/* Se l'elemento  presente... */

				if (current_founded->status == ACTIVE){
						current_founded->status = BLOCKED;
				}
				else{
						current_founded->status = ACTIVE;
				}

				return(1);
		}
		else return(-1);
}
