#define _SIGNAL_AND_EXECS_C

#ifndef _PSH_MAIN
#include "patty_shell.c"
#endif

#ifndef _PPROCS_STRUCT_FUNCTION_C
#include "pprocs_struct_function.c"
#endif

#ifndef _SIGNAL_H
#include <signal.h>
#endif

#ifndef _STDIO_H
#include <stdio.h>
#endif

#ifndef _SYS_WAIT_H
#include <sys/wait.h>
#endif

#define TERMINATED	10
#define STOPPED		20

/*---------------------------------------------------exit_status_evaluate*/

/*	"Visualizza" lo stato di uscita di un processo figlio.
	Stampa anche il PID del corrispondente processo.

	INPUT:
	1) PID del processo
	2) Intero (ritornato da una WAIT) dello stato del processo

	OUTPUT:
	"TERMINATED" se il processo risulta appunto terminato,
	altrimenti "STOPPED". 
	Per la definizione delle costanti, consultare la testa di questo file */

int exit_status_evaluate(pid_t pid, int exit_status)

{
	if (WIFSTOPPED(exit_status)) { /* Se il processo risulta STOPPATO...*/
			printf("Process %d STOPPED. Signal number: %d\n", pid, WSTOPSIG(exit_status));
			return(STOPPED);
	}
	
	if (WIFEXITED(exit_status)) { /* ...altrimenti, se risulta terminato...*/
			printf("Process %d normal terminated. Exit status: %d\n", pid, WEXITSTATUS(exit_status));
			return(TERMINATED);
	}
	else
			if (WIFSIGNALED(exit_status)) { /* ...ed infine se risulta terminato da un segnale */
					printf("Process %d ABNORMAL terminated. Signal number: %d%s\n", pid, WTERMSIG(exit_status),
/* questa definizione serve a visualizzare il messaggio di un eventuale CORE DUMP */
#ifdef WCOREDUMP
									WCOREDUMP(exit_status) ? " (core dumped)" : "");
#else
					"");
#endif
					return(TERMINATED);
			}
}
/*---------------------------------------------------*/

/*---------------------------------------------------psh_signal_handler*/

/* SIGNAL HANDLER per la Patty Shell. */
static void psh_signal_handler(int Signal)
{
	extern pid_t fg_pid;		/* Variabili esterne dichiarate nel MAIN_FILE della PSH */
	extern pid_t sh_pid;
	extern logical sigtstp;

	pid_t target;				/* PID del processo a cui riinviare SIGINT */
	
	if(fg_pid != sh_pid){					
		if(Signal == SIGTSTP || Signal == SIGINT || Signal == SIGQUIT){
			kill(fg_pid, Signal);		/* "Rimbalzo" del segnale */
			
			printf("Signal Rebounded to process %d\n", fg_pid);
			psignal(Signal, "--->");
			
			fg_pid=sh_pid;				/* Aggiornamento variabili: si ripristina lo stato originale. */
			
			if (Signal == SIGTSTP) sigtstp=TRUE;	/* Se  un segnale SIGTSTP setta sigtstp a TRUE
														   		PSH sapr cos di dover tracciare un processo stoppato.
														   		Questa implementazione  molto poco elegante, 
																	ma nella documentazione spiegher perch...*/
			else sigtstp=FALSE;
		}
	}
	else{
			  if (Signal == SIGINT) {					/* Permette di digitare il PID del processo a cui
																	rimbalzare il segnale */
						 show_pprocs(PPROCS_STRUCT_REFERENCE,ALL);
						 printf("Digit PID to rebound SIGINT: ");
						 scanf("%d", &target);
						 kill(target,SIGINT);
			  }
			  else
						 psignal(Signal, "\nIgnored by PSH ");		/* il segnale viene "evitato" dalla PSH */	
	}
}
/*---------------------------------------------------*/

/*---------------------------------------------------psh_signal_redirection*/

/* Esegue solo l'assegnazione dei nuovi SIGNAL HANDLER */
void psh_signal_redirection(void)
{
	struct sigaction *sigchld_act;			/* Questa struttura serve a "sig_action" 
											   per la descrizione completa dell'handler*/
	
	sigchld_act=(struct sigaction*)malloc(sizeof(struct sigaction));
	
	sigchld_act->sa_handler=psh_signal_handler;
		/* In realt sarebbe necessario, per evitare che 
			pi SEGNALI arrivino al processo contemporaneamente,
		 	che si mascheri il segnale mentre viene gestito dall'HANDLER. 
		  	In realt, a differenza della funzione SIGNAL, 
			questo viene gi eseguito dalla funzione SIGACTION */
	
	/* Assegnazione dei Signal Handler: */
	sigaction(SIGTSTP, sigchld_act, NULL);
	sigaction(SIGINT, sigchld_act, NULL);
	sigaction(SIGQUIT, sigchld_act, NULL);
	
	printf("\nSignal Redirection DONE.\n");
}
/*---------------------------------------------------*/

/*---------------------------------------------------wait_processes*/
/* Attesa eventuali processi terminati */
int wait_childs(void)
{
		pid_t any_pid=0;
		int status;

							/* ...finch ci sono processi terminati ma non TRACCIATI...*/
		while ( ((any_pid=waitpid(WAIT_ANY, &status, WNOHANG)) != 0) && (any_pid != -1) ){
				exit_status_evaluate(any_pid, status);
				remove_pprocs(any_pid, PPROCS_STRUCT_REFERENCE);
		}		
}
/*---------------------------------------------------*/

/*--------------------------------------------------add_and_wait */
/* (1)Aggiunge un processo alla struttura PPROCS,
	(2)si mette in wait per il suddetto processo
	(3)analizza l'eventuale stato di terminazione (o stop) del processo
			aggiornando di conseguenza la struttura PPROCS

	E' utilizzata per l'esecuzione SINCRONA dei processi */
/* INPUT:
		proc_name = nome processo da inserire nella struttura PPROCS
		pid = PID del processo */
void add_and_wait(char *proc_name, pid_t pid)
{
   extern logical sigtstp;
   extern PPROCS *PPROCS_STRUCT_REFERENCE;
   extern pid_t sh_pid;
   extern pid_t fg_pid;

   int cstatus;
   int stopORterm=0;

   add_pprocs(proc_name, pid, sh_pid, ACTIVE, PPROCS_STRUCT_REFERENCE);
   fg_pid=pid;

   waitpid(pid, &cstatus, 0);
   if (sigtstp == TRUE)		/* Se risulta essere stato stoppato da CTRL+Z...*/
      waitpid(pid, &cstatus, WUNTRACED);
   sigtstp=FALSE;
										
   stopORterm=exit_status_evaluate(pid, cstatus);		/* Valuta lo stato di uscita */
   
   if (stopORterm == TERMINATED)
      remove_pprocs(pid, PPROCS_STRUCT_REFERENCE);		/* o il processo viene rimosso da PPROCS... */
   else
		if (stopORterm == STOPPED)
	 		ch_status_pprocs(pid, PPROCS_STRUCT_REFERENCE);		/* ...o viene aggiornato lo stato del processo in PPROCS */
	fg_pid=sh_pid;
}

/*-----------------------------------------execute_and_wait */
/* Esegue un semplice FORK-EXEC; 
	attende la terminazione del processo;
	valuta lo stato di terminazione del processo*/

/* INPUT: 
	"argv_p"  un "array di stringhe" che contiene la lista di argomenti
		del processo da eseguire. */
/* OUTPUT:
	Ritorna "-1" in caso di errore. Altrimenti "1" */
	
int execute_and_wait(char **argv_p)
{
	
		  pid_t new_pid;		  
		  if ( (new_pid=fork()) < 0) {
					 perror("\n---Fork ERROR---\n");
					 return(-1);
		  }
		  else {
					 if (new_pid > 0) {
								add_and_wait(argv_p[0], new_pid);
					 }
					 else {
								if (new_pid == 0) {
										  execvp(argv_p[0], argv_p);
										  perror("\n---Exec ERROR---\n");
										  return(-1);
								}
					 }
		  }
		  return(1);
}
/*----------------------------------------------------------*/

/*------------------------------------------execute_background */
/*	Esegue un semplice FORK-EXEC;
	Non attende che il processo termini.*/

/* INPUT: 
	"argv_p"  un "array di stringhe" che contiene la lista di argomenti
	del processo da eseguire. */
/* OUTPUT:
	Ritorna "-1" in caso di errore. Altrimenti "1" */

int execute_background(char **argv_p)
{
		  extern pid_t sh_pid;
		  extern PPROCS* PPROCS_STRUCT_REFERENCE;
		  
		  pid_t new_pid;
		
		  if ( (new_pid=fork()) < 0) {
					 perror("\n---Fork ERROR---\n");
					 return(-1);
		  }
		  else {
					 if (new_pid > 0) {
								add_pprocs(argv_p[0],new_pid,sh_pid,ACTIVE,PPROCS_STRUCT_REFERENCE);
					 }
					 else {
								if (new_pid == 0) {
										  execvp(argv_p[0], argv_p);
										  perror("\n---Exec ERROR---\n");
										  return(-1);
								}
					 }
		  }
		  return(1);
}
